import java.io.*;

//import no.rwr.render.*;
import no.rwr.image.*;
import java.awt.image.*;
import java.awt.color.*;
import java.awt.Point;
import java.awt.Transparency;

import no.rwr.x.*;
import no.rwr.xutil.*;
import no.rwr.xutil.event.*;
import no.rwr.image.codec.libjpeg.JPEGImageDecoder;

public class Viewer implements WindowCloseRequestListener{

    static final boolean VERBOSE = true;

    public static void main(String[] args) throws IOException {
	String filename = null;
	try {
	    filename = args[0];
	} catch (ArrayIndexOutOfBoundsException aioobe) {
	    System.err.println("Viewer: image file name missing");
	    System.err.println("Usage: Viewer jpeg_image_file");
	    System.exit(1);
	}
	new Viewer(filename);
    }

    no.rwr.xutil.Frame frame;

    public Viewer(String filename) throws IOException {
	WritableRaster inputRaster;
	ColorModel inputCM;
	

	/*
	// Read TGA image
	TgaParser tp = new TgaParser(new FileInputStream(filename));
	if (VERBOSE) System.out.println("TGA parsing done");
	inputRaster = tp.wRaster;
	inputCM = tp.colorModel;             // Color model
	*/

	// Read JPEG image
	JPEGImageDecoder jid =
	    new JPEGImageDecoder(new FileInputStream(filename));
	BasicRasterImageConsumer consumer = new BasicRasterImageConsumer();
	jid.startProduction(consumer);
	inputRaster = consumer.getImage().getRaster();
	inputCM = consumer.getImage().getColorModel();



	// ASSUMING:

	ColorSpace sRGB = ColorSpace.getInstance(ColorSpace.CS_sRGB);
	int[] bits = { 8, 8, 8 };
	


	// Retrieve information about image (data and format)
	SampleModel inputSM = inputRaster.getSampleModel(); // Sample model

	// Decide where to display it
	Display display = new Display();
	Screen screen = display.getDefaultScreen();
	no.rwr.x.Window root = screen.getRootWindow();

	// FIXME: Make a point of finding the best visual
	Visual visual = screen.getRootVisual();

	no.rwr.xutil.Image image =
	    getImage(visual, inputCM, inputSM, screen, inputRaster);

	// Create pixmap with image
	Pixmap pixmap;
	pixmap = image.getPixmap();

	if (pixmap == null) {
	    throw new RuntimeException("image.getPixmap() returned a " +
				       "null reference for some odd " +
				       "reason. This should not happen.");
	}

	

	createViewerFrame(pixmap,
			  image.getWidth(), image.getHeight(),
			  visual, root);

	// Run event loop
	Dispatcher dispatcher = new Dispatcher(display);
	dispatcher.run();
    }

    public void createViewerFrame(Pixmap pixmap, int width, int height,
				  Visual visual, no.rwr.x.Window parent) 
    {
	// Create window and display image
	WindowAttributes frameAttr = new WindowAttributes();

	frameAttr.setBackground(pixmap);
	frameAttr.setVisual(visual);

	// Do not allow window to be larger than the pixmap, since this
	// will show an ugly tiling effect.
	WMSizeHints sizeHints = new WMSizeHints();
	sizeHints.setMaxSize(width, height);

	frame = new no.rwr.xutil.Frame();
	
	// FIXME: Why can't frame.getWindow() be used here?

	frame.setTitle("Image viewer hack");
	frame.setSize(width, height);
	frame.setNormalSizeHints(sizeHints);
	frame.setParent(parent);
	frame.setAttributes(frameAttr);

	frame.addWindowCloseRequestListener(this);

	frame.setVisible(true);
    }
    


    public no.rwr.xutil.Image getImage(Visual visual, ColorModel cm,
				       SampleModel sm,
				       Screen screen, Raster raster) {
	try {
	    return getImageDirectly(visual, cm, sm, raster);
	} catch (IllegalArgumentException iae) {
	    return getRenderedImage(visual, cm, sm, raster, screen);
	}
    }

    public no.rwr.xutil.Image getImageDirectly(Visual visual, ColorModel cm,
					       SampleModel sm, Raster raster) {
	XImageFactory factory = new XImageFactory(visual, cm, sm);
	return new no.rwr.xutil.Image(factory, raster);
    }

    public  no.rwr.xutil.Image getRenderedImage(Visual visual,
						ColorModel cm,
						SampleModel sm,
						Raster raster,
						Screen screen) {

	
	if (VERBOSE) System.err.println("direct factory failed, " +
					"trying renderer");

	no.rwr.xutil.Image image = new no.rwr.xutil.Image(visual,
				       screen.getDefaultColormap(),
				       raster.getWidth(),
				       raster.getHeight());


	XImageFactory factory = image.getXImageFactory();



	//
	// Ask the Renderers utility class to find a renderer that
	// can convert from the data format of the TGA image to
	// the data format supported by the display.
	
	RasterOp renderer =
	    Renderers.getInstance().createRenderer(cm, sm,
						   factory.getColorModel(),
						   factory.getSampleModel());

	if (renderer == null) {
	    String msg =
		"It was not possible to obtain a renderer that can " +
		"convert the input raster (" + raster + ") to the image " +
		"format (" + image + ") supported by the screen visual (" +
		visual + ")";
	    throw new UnsupportedOperationException(msg);
	}

	if (VERBOSE) System.out.println("Rendering image using " + renderer);

	

	WritableRaster imageRaster = image.getWritableRaster();
	renderer.filter(raster, imageRaster);

	if (VERBOSE) System.out.println("Rendering finished");
	return image;
    }

    

    public void windowCloseRequested(WindowCloseRequestEvent wcre) {
	if (VERBOSE) System.out.println("Closing window per request");
	frame.setVisible(false);
	System.exit(0);

    }
}
