package java.awt;

public class Color implements 
    //Paint,
    //Tansparency,
    java.io.Serializable {

    public static final Color white     = new Color(0xffffffff, true);
    public static final Color lightGray = new Color(0xffc0c0c0, true);
    public static final Color gray      = new Color(0xff808080, true);
    public static final Color darkGray  = new Color(0xff404040, true);
    public static final Color black     = new Color(0xff000000, true);
    public static final Color red       = new Color(0xffff0000, true);
    public static final Color pink      = new Color(0xffffafaf, true);
    public static final Color orange    = new Color(0xffffc800, true);
    public static final Color yellow    = new Color(0xffffff00, true);
    public static final Color green     = new Color(0xff00ff00, true);
    public static final Color magenta   = new Color(0xffff00ff, true);
    public static final Color cyan      = new Color(0xff00ffff, true);
    public static final Color blue      = new Color(0xff0000ff, true);

    private int argb;

    public Color(int r, int g, int b) {
	this(r, g, b, 0xff);
    }

    public Color(int r, int g, int b, int a) {
	argb = (a << 24) | (r << 16) | (g << 8) | b;
    }

    public Color(int rgb) {
	argb = rgb | 0xff000000;
    }

    public Color(int rgba, boolean hasAlpha) {
	argb = hasAlpha ? rgba : (rgba | 0xff000000);
    }
    
    public Color(float r, float g, float b) {
	this(r, g, b, 1.0F);
    }

    public Color(float r, float g, float b, float a) {
	this((int) (r*255F + 0.5F), 
	     (int) (g*255F + 0.5F),
	     (int) (b*255F + 0.5F),
	     (int) (a*255F + 0.5F));
    }
    
    //public Color(ColorSpace cspace, float[] components, float alpha);

    public int getRed() {
	return (argb >> 16) & 0xff;
    }

    public int getGreen() {
	return (argb >> 8) & 0xff;		
    }

    public int getBlue() {
	return argb & 0xff;
    }

    public int getAlpha() {
	return (argb >> 24) & 0xff;
    }

    public int getRGB() {
	return argb;
    }

    static final int BRIGHT_STEP = 0x30;

    public Color brighter() {
	return new Color(Math.min(255, getRed()   + BRIGHT_STEP),
			 Math.min(255, getGreen() + BRIGHT_STEP),
			 Math.min(255, getBlue()  + BRIGHT_STEP),
			 getAlpha());
    }
    
    public Color darker() {
	return new Color(Math.max(0, getRed()   - BRIGHT_STEP),
			 Math.max(0, getGreen() - BRIGHT_STEP),
			 Math.max(0, getBlue()  - BRIGHT_STEP),
			 getAlpha());
    }
    
    public int hashCode() {
	return argb;
    }
    
    // Not necessary using the above hashCode()
    //public boolean equals(Object obj);

    public String toString() {
	return getClass().getName() +
	    "[r=" + getRed() +
	    ",g=" + getGreen() +
	    ",b=" + getBlue() +
	    "]";
    }
    
    //public static Color decode(String nm) throws NumberFormatException;
    //public static Color getColor(String nm);
    //public static Color getColor(String nm, Color v)
    //public static Color getColor(String nm, int v)
    /*
    public static int HSBtoRGB(float hue, float saturation,
			       float brightness);
    */

    
    //public static float[] RGBtoHSB(int r, int g, int b, float[] hsbvals);
    //public static Color getHSBColor(float h, float s, float b);
    //public float[] getRGBComponents(float[] compArray);
    //public float[] getRGBColorComponents(float[] compArray);
    //public float[] getComponents(float[] compArray);
    //public float[] getColorComponents(float[] compArray);
    //public float[] getComponents(ColorSpace cspace, float[] compArray);
    //public float[] getColorComponents(ColorSpace cspace, float[] compArray);
    //public ColorSpace getColorSpace();
    /*
    public PaintContext createContext(ColorModel cm, Rectangle r,
				      Rectangle2D r2d,
				      AffineTransform xform,
				      RenderingHints hints);
    */
    //public int getTransparency();
}
