package javax.swing;
import java.io.Serializable;
import java.util.Vector;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;


public class DefaultButtonModel implements ButtonModel, Serializable {

    boolean isArmed = false;
    boolean isSelected = false;
    boolean isEnabled = true;
    boolean isPressed = false;
    boolean isRollover = false;

    int mnemonic;
    String command;

    // FIXME... don't use Vector for this.
    Vector actionListeners = new Vector();
    Vector itemListeners = new Vector();
    Vector changeListeners = new Vector();

    public boolean isArmed() {
	return isArmed;
    }

    public boolean isSelected() {
	return isSelected;
    }

    public boolean isEnabled() {
	return isEnabled;
    }

    public boolean isPressed() {
	return isPressed;
    }

    public boolean isRollover() {
	return isRollover;
    }

    public void setArmed(boolean armed) {
	if (isArmed == armed) return;
	isArmed = armed;
	fireChangeEvent();
    }

    public void setSelected(boolean selected) {
	if (isSelected == selected) return;
	isSelected = selected;
	fireChangeEvent();
    }

    public void setEnabled(boolean enabled) {
	if (isEnabled == enabled) return;
	isEnabled = enabled;
	fireChangeEvent();
    }

    public void setPressed(boolean pressed) {
	if (isPressed == pressed) return;
	isPressed = pressed;

	if (isPressed()) isArmed = true;

	fireChangeEvent();

	if ((!isPressed()) && isArmed()) fireActionEvent();
    }

    public void setRollover(boolean rollover) {
	if (isRollover == rollover) return;
	isRollover = rollover;
	fireChangeEvent();
    }

    public void setMnemonic(int key) {
	mnemonic = key;
    }

    public int getMnemonic() {
	return mnemonic;
    }

    public void setActionCommand(String string) {
	command = string;
    }

    public String getActionCommand() {
	return command;
    }

    //public void setGroup(ButtonGroup group);

    public void addActionListener(ActionListener listener) {
	actionListeners.addElement(listener);
    }

    public void removeActionListener(ActionListener listener) {
	actionListeners.removeElement(listener);
    }

    public void addItemListener(ItemListener listener) {
	itemListeners.addElement(listener);
    }

    public void removeItemListener(ItemListener listener) {
	itemListeners.removeElement(listener);
    }
    
    public void addChangeListener(ChangeListener listener) {
	changeListeners.addElement(listener);
    }

    public void removeChangeListener(ChangeListener listener) {
	changeListeners.removeElement(listener);
    }

    protected void fireChangeEvent() {
	int size = changeListeners.size();
	ChangeListener[] listeners = new ChangeListener[size];
	changeListeners.copyInto(listeners);

	ChangeEvent event = new ChangeEvent(this);

	for (int i=0; i<size; i++) {
	    listeners[i].stateChanged(event);
	}
    }

    private void fireActionEvent() {
	int size = actionListeners.size();
	ActionListener[] listeners = new ActionListener[size];
	actionListeners.copyInto(listeners);

	ActionEvent event =
	    new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
			    getActionCommand());
	

	for (int i=0; i<size; i++) {
	    listeners[i].actionPerformed(event);
	}

    }
}
