package no.rwr.data;
import java.awt.image.*;

public class Buffers {

    public static DataBuffer createBuffer(int dataType, Object data,
					  int size) {
	return createBuffer(dataType, data, size, 1);
    }

    public static DataBuffer createBuffer(int dataType, Object data,
					  int size, int numBanks) {
	if (data == null)
	    return createBuffer(dataType, size, numBanks);
	return createBufferFromData(dataType, data, size, numBanks);
	
    }

    public static DataBuffer createBuffer(int dataType, int size) {
	return createBuffer(dataType, size, 1);
    }

    public static DataBuffer createBuffer(int dataType, int size,
					  int numBanks) {
	switch (dataType) {
	case DataBuffer.TYPE_BYTE:
	    return new DataBufferByte(size, numBanks);
	case DataBuffer.TYPE_USHORT:
	    return new DataBufferUShort(size, numBanks);
	case DataBuffer.TYPE_INT:
	    return new DataBufferInt(size, numBanks);
	default:
	    throw new UnsupportedOperationException();
	}
    }

    public static DataBuffer createBufferFromData(int dataType, Object data,
						  int size) {
	return createBufferFromData(dataType, data, size, 1);
    }

    public static DataBuffer createBufferFromData(int dataType, Object data,
						  int size, int numBanks) {
	switch (dataType) {
	case DataBuffer.TYPE_BYTE:
	    return new DataBufferByte((byte[]) data, size, numBanks);
	case DataBuffer.TYPE_USHORT:
	    return new DataBufferUShort((short[]) data, size, numBanks);
	case DataBuffer.TYPE_INT:
	    return new DataBufferInt((int[]) data, size, numBanks);
	default:
	    throw new UnsupportedOperationException();
	}
    }

    public static Object getData(DataBuffer buffer) {
	if (buffer instanceof DataBufferByte)
	    return ((DataBufferByte) buffer).getData();
	if (buffer instanceof DataBufferUShort)
	    return ((DataBufferUShort) buffer).getData();
	if (buffer instanceof DataBufferInt)
	    return ((DataBufferInt) buffer).getData();
	throw new ClassCastException("Unknown data buffer type");
    }

    
    public static int smallestAppropriateTransferType(int bits) {
	if (bits <= 8) {
	    return DataBuffer.TYPE_BYTE;
	} else if (bits <= 16) {
	    return DataBuffer.TYPE_USHORT;
	} else if (bits <= 32) {
	    return DataBuffer.TYPE_INT;
	} else {
	    return DataBuffer.TYPE_UNDEFINED;
	}
    }
}
