package no.rwr.image;

import java.awt.image.*;
import java.awt.GraphicsConfiguration;
import java.util.*;

public class BasicRasterImageConsumer implements RasterImageConsumer {

    static final int VERBOSE_LEVEL = 0;

    int width, height;
    Hashtable properties;
    ColorModel colorModel;
    int hints;
    BufferedImage image;
    WritableRaster raster;
    GraphicsConfiguration config;

    public BasicRasterImageConsumer() {};

    public BasicRasterImageConsumer(GraphicsConfiguration config) {
	this.config = config;
    };



    public BufferedImage getImage() {
	return image;
    }

    public void setDimensions(int width,
			      int height) {
	this.width  = width;
	this.height = height;
	if (VERBOSE_LEVEL > 0)
	    System.out.println("dimensions=" + width + "x" + height);
    }

    public void setProperties(Hashtable props) {
	properties = props;
    }

    public void setColorModel(ColorModel model) {
	colorModel = model;
    }

    public void setHints(int hintflags) {
	hints = hintflags;
	raster =
	    colorModel.createCompatibleWritableRaster(width, height);

	if (config != null) {
	    image = config.createCompatibleImage(width, height);
	} else {
	    image = new BufferedImage(colorModel, raster,
				      false, // not premultiplied
				      properties);
	}
    }


    public void setPixels(int x,
			  int y,
			  int w,
			  int h,
			  ColorModel model,
			  byte[] pixels,
			  int off,
			  int scansize) {
	throw new UnsupportedOperationException("not implemented");
    }
    
    public void setPixels(int x,
			  int y,
			  int w,
			  int h,
			  ColorModel model,
			  int[] pixels,
			  int off,
			  int scansize) {
	throw new UnsupportedOperationException("not implemented");
    }

    public void imageComplete(int status) {
	// NOP
    }

    public void setPixels(Raster segment) {
	if (VERBOSE_LEVEL > 1)
	    System.out.println(segment.getBounds());

	


	if (VERBOSE_LEVEL > 2) {
	    int startY = segment.getMinY();
	    int endY = startY + segment.getHeight();
	    int startX = segment.getMinX();
	    int endX = startX + segment.getWidth();
	    for (int y = startY; y<endY; y++) {
		for (int x = startX; x<endX; x++) {
		    System.out.println("getSample("+x+","+y+",{0,1,2}) = {" +
				       segment.getSample(x,y,0) + ", " +
				       segment.getSample(x,y,1) + ", " +
				       segment.getSample(x,y,2) + "}");
		}
	    }
	}
	


	image.setData(segment);
    }

}
