package no.rwr.image.hermes;

import java.awt.image.*;
import no.rwr.image.*;
import no.rwr.util.ArrayUtil;
import no.rwr.util.BitMaskExtent;
import gnu.gcj.RawData;

public class HermesFormat {

    IndexColorModel palette;

    public HermesFormat(ColorModel cm, SampleModel sm) {

	if (cm instanceof ComponentColorModel) {
	    initComponentFormat((ComponentColorModel) cm,
				(ComponentSampleModel) sm);
	    return;
	}

	if (cm instanceof PackedColorModel) {
	    initPackedFormat((PackedColorModel) cm,
			     (SinglePixelPackedSampleModel) sm);
	    return;
	}

	if (cm instanceof IndexColorModel) {
	    initIndexFormat((IndexColorModel) cm,
			     (ComponentSampleModel) sm);
	    return;
	}

	String msg =
	    "color model (" + cm.getClass().getName() + ") not supported";
	throw new IllegalArgumentException(msg);
    }

    public void initComponentFormat(ComponentColorModel ccm,
				    ComponentSampleModel csm) {
	int numComponents = ccm.getNumColorComponents();
	if (numComponents != 3)
	    throw new IllegalArgumentException("Wrong number of color " +
					       "components.");
	
	byte bitsPerComponent = (byte)
	    DataBuffer.getDataTypeSize(csm.getDataType());
	int pixelStride = csm.getPixelStride();
	int bitsPerPixel = bitsPerComponent * pixelStride;
	int[] bandOffset = csm.getBandOffsets();
	
	BitMaskExtent bme = new BitMaskExtent();
	bme.bitWidth = bitsPerComponent;
	int[] masks = new int[4];
	
	for (int b=0; b<3; b++) {
	    int shift = bandOffset[b]*bitsPerComponent;
	    if ((shift+bitsPerComponent)>32)
		throw new IllegalArgumentException("Samples too spread");
	    bme.leastSignificantBit = (byte) shift;
	    masks[b] = (int) (bme.toMask() & 0xffffffff);
	}
	    
	init(masks[0], masks[1], masks[2], masks[3],
	     bitsPerPixel, false,
	     false, 0);
    }
    

    public void initPackedFormat(PackedColorModel pcm,
				 SinglePixelPackedSampleModel sppsm) {
	int bitsPerPixel =
	    DataBuffer.getDataTypeSize(sppsm.getDataType());
	int[] masks = pcm.getMasks();
	boolean hasAlpha = pcm.hasAlpha();
	
	init(masks[0], masks[1], masks[2], hasAlpha ? masks[3] : 0,
	     bitsPerPixel, false,
	     false, 0);
    }
	
    public void initIndexFormat(IndexColorModel icm,
				ComponentSampleModel csm) {
	palette = icm;
	int bitsPerPixel = DataBuffer.getDataTypeSize(csm.getDataType());
	boolean isIndexed = true;
	int colorKey = icm.getTransparentPixel();
	boolean hasColorKey = (colorKey != -1);
	init(0, 0, 0, 0,
	     bitsPerPixel, isIndexed,
	     hasColorKey, colorKey);
    }
    
    public native void init(int rMask, int gMask, int bMask, int aMask, 
			    int bitsPerPixel, boolean isIndexed, 
			    boolean hasColorKey, int colorKey);

    RawData structure;

    public int[] getRGBAPalette() {
	final int HERMES_PALETTE_SIZE = 256;
	if (palette == null) return null;
	int[] rgbaArray = new int[HERMES_PALETTE_SIZE];
	palette.getRGBs(rgbaArray);
	return rgbaArray;
    }
    
}
