package no.rwr.plaf;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;


public class GButtonUI extends ButtonUI implements ChangeListener {

    static final Border raisedBorder = new BevelBorder(BevelBorder.RAISED,
						       Color.white,
						       Color.lightGray,
						       Color.black,
						       Color.gray);
    static final Border loweredBorder = new BevelBorder(BevelBorder.LOWERED,
							Color.white,
							Color.lightGray,
							Color.black,
							Color.gray);

    public static ComponentUI createUI(JComponent button) {
	return new GButtonUI((AbstractButton) button);
    }

    AbstractButton button;

    public GButtonUI(AbstractButton button) {
	this.button = button;
    }
    public void installUI(JComponent c) {
	button.addMouseListener(GButtonListener.INSTANCE);
	button.getModel().addChangeListener(this);

	// make the default UI update() method fill background...
	button.setOpaque(true);
	button.setBackground(Color.lightGray);
	updateBorder();
    }
    
    public void uninstallUI(JComponent c) {
	button.removeMouseListener(GButtonListener.INSTANCE);
	button.getModel().removeChangeListener(this);
    } 


    public void stateChanged(ChangeEvent ce) {
	updateBorder();
    }
    
    void updateBorder() {
	boolean isPressed = button.getModel().isPressed();
	
	Border border =
	    button.getModel().isPressed() ? loweredBorder : raisedBorder;
	
	button.setBorder(border);
    }

    public void paint(Graphics gfx, JComponent c) {
	
	// temporary scribble to acknowledge presence
	String text = ((AbstractButton) c).getText();

	if (text == null) return;
	
	Font f = new Font("helvetica", Font.PLAIN, 12);
	gfx.setFont(f);
	FontMetrics fm = gfx.getFontMetrics();
	int y = (c.getHeight() - fm.getHeight()) / 2 + fm.getAscent();
	int x = (c.getWidth() - fm.stringWidth(text)) / 2;

	gfx.setColor(Color.black);
   
	
	gfx.drawString(text, x, y);

    }


    // FIXME: only temporary stub
    public Dimension getPreferredSize(JComponent c) {
	return new Dimension(60, 24);
    }

}


/** Translates mouse event on an AbstractButton to button module
    manipulations.  Note: only one instance is ever needed.  */
class GButtonListener extends MouseAdapter {
    public final static MouseListener INSTANCE = new GButtonListener();
    
    private GButtonListener() {}

    public void mousePressed (MouseEvent event) {
	AbstractButton button = (AbstractButton) event.getSource();
	button.getModel().setPressed(true);
    }

    public void mouseReleased (MouseEvent event) {
	AbstractButton button = (AbstractButton) event.getSource();
	ButtonModel model = button.getModel();

	int x = event.getX();
	int y = event.getY();
	if (!button.contains(x, y)) {
	    model.setArmed(false);
	}
	

	model.setPressed(false);
    }
}
