
#include <vector>

#include <X11/Xlib.h>

#include <gcj/cni.h>
#include <gnu/gcj/RawData.h>
#include <java/lang/String.h>
#include <java/awt/Rectangle.h>

#include "no/rwr/x/Display.h"
#include "no/rwr/x/XID.h"
#include "no/rwr/x/Drawable.h"
#include "no/rwr/x/Font.h"
#include "no/rwr/x/XImage.h"
#include "no/rwr/x/XException.h"
#include "no/rwr/x/Clip.h"
#include "no/rwr/x/GC.h"


typedef java::awt::Rectangle AWTRect;
typedef JArray<AWTRect*> AWTRectArray;
typedef vector<XRectangle> XRectVector;


gnu::gcj::RawData* no::rwr::x::GC::getValues() {
    if (values == 0) {
	XGCValues* vals = new XGCValues;
	values = reinterpret_cast<gnu::gcj::RawData*>(vals);
    }

    return values;
}


void no::rwr::x::GC::copyStructures(GC* to) {
    to->structure = 0;
    to->values = 0;

    if (values == 0) return;
    
    if (structure == 0) {
	::XGCValues* vals = (::XGCValues*) values;
	::XGCValues* valsCopy = new XGCValues;
	*valsCopy = *vals;
	to->values = reinterpret_cast<gnu::gcj::RawData*>(valsCopy);
    } else {
	
	/* Copying the values we've got is not good enough. Must ask
	   Xlib to copy what it got. */
	
	setMask |= valueMask;

	to->valueMask = valueMask;

	/*
	// The Xlib uses optimized copy if exactly these bits are selected.
	    GCFunction        | GCPlaneMask         | GCForeground  | 
	    GCBackground      | GCLineWidth         | GCLineStyle   | 
	    GCCapStyle        | GCJoinStyle         | GCFillStyle   | 
	    GCFillRule        | GCTile              | GCStipple     |
	    GCTileStipXOrigin | GCTileStipYOrigin   | GCFont        |
	    GCSubwindowMode   | GCGraphicsExposures | GCClipXOrigin |
	    GCClipYOrigin     | GCDashOffset        | GCArcMode;
	*/
	Display* display = targetType->getDisplay();
	::Display* dpy = (::Display*) (display->display);
	::GC gc = (::GC) getStructure();
	
	XGCValues* vals = new XGCValues;
	to->values = reinterpret_cast<gnu::gcj::RawData*>(vals);
	
	Status result = XGetGCValues(dpy, gc, to->valueMask, vals);
	
	if (result == 0) {
	    jstring msg = JvNewStringLatin1("XGetGCValues failed");
	    JvThrow(new no::rwr::x::XException(msg));
	}
    }
}


/** Perform lazy creation or updating if necessary. */
gnu::gcj::RawData* no::rwr::x::GC::getStructure() {

    // Short circuit
    if ((structure != 0) && (valueMask == 0) && clipUpToDate)
	return structure;

    ::GC gc = (::GC) structure;
    
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::XGCValues* vals = (::XGCValues*) values;
    
    if (gc == 0) {
	/* We don't use getValues() since we don't care that vals ==
	   0, since if vals == 0 then valueMask == 0.  */
	gc = XCreateGC(dpy, drawableXID, valueMask, vals);
	structure = reinterpret_cast<gnu::gcj::RawData*>(gc);
	valueMask = 0; // All values are now up to date
    } else if (valueMask != 0) {	
	// we know vals != null, since valueMask != 0
	XChangeGC(dpy, gc, valueMask, vals);
	setMask |= valueMask;
	valueMask = 0; // All values are now up to date
    }

    if (!clipUpToDate) {
	XRectVector* xrectvector = (XRectVector*) (clip->xrects);
	int numRect = xrectvector->size();
	
	int originX = 0;
	int originY = 0;
	int ordering = Unsorted;
	int status = XSetClipRectangles(dpy, gc, originX, originY,
					xrectvector->begin(), numRect,
					ordering);
	switch (status) {
	case BadAlloc: case BadGC: case BadMatch: case BadValue:
	    JvThrow(new XException(display, status));
	}
	clipUpToDate = true;
    }

    return structure;
}

void no::rwr::x::GC::dispose() {
    if (structure != 0) {
	Display* display = targetType->getDisplay();
	::Display* dpy = (::Display*) (display->display);
	::GC gc = (::GC) structure;
	XFreeGC(dpy, gc);
	structure = 0;
    }
    delete values;
    values = 0;

}

void no::rwr::x::GC::setForeground(jlong pixel) {
    ::XGCValues* vals = (::XGCValues*) getValues();;

    vals->foreground = pixel;
    valueMask = valueMask | GCForeground;
}

void no::rwr::x::GC::setFont(Font* font) {
    ::XGCValues* vals = (::XGCValues*) getValues();

    vals->font = font->getXID();
    valueMask = valueMask | GCFont;
}

void no::rwr::x::GC::drawString(jstring text, jint x, jint y) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();

    /*
    jint length = text->length();
    jchar* txt = JvGetStringChars(text);

    XChar2b xwchars[length];
    
    // FIXME: Add convertion and caching

    for (int i=0; i<length; i++) {
	XChar2b* xc = &(xwchars[i]);
	jchar jc = txt[i];
	xc->byte1 = jc & 0xff;
	xc->byte2 = jc >> 8;
    }

     XDrawString16(dpy, drawableXID, gc, x, y, xwchars, length);
    */

    // ALTERNATIVE:
    int len = JvGetStringUTFLength(text);
    char ctxt[len+1];
    JvGetStringUTFRegion(text, 0, text->length(), ctxt);
    ctxt[len] = '\0';
    XDrawString(dpy, drawableXID, gc, x, y, ctxt, len);
}

void no::rwr::x::GC::drawLine(jint x1, jint y1, jint x2, jint y2) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    XDrawLine(dpy, drawableXID, gc, x1, y1, x2, y2);
}

void no::rwr::x::GC::drawRectangle(jint x, jint y, jint w, jint h) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    XDrawRectangle(dpy, drawableXID, gc, x, y, w, h);
}

void no::rwr::x::GC::fillRectangle(jint x, jint y, jint w, jint h) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    XFillRectangle(dpy, drawableXID, gc, x, y, w, h);
}

void no::rwr::x::GC::clearArea(jint x, jint y, jint w, jint h,
			       jboolean exposures) {
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    //::GC gc = (::GC) getStructure();
    XClearArea(dpy, drawableXID, x, y, w, h,
	       exposures ? True : False);
}


void no::rwr::x::GC::putImage(XImage* image,
			      jint srcX, jint srcY,
			      jint destX, jint destY,
			      jint width, jint height) {
    
    Display* display = targetType->getDisplay();
    ::Display* dpy = (::Display*) (display->display);
    ::Drawable drawableXID = targetType->getXID();
    ::GC gc = (::GC) getStructure();
    ::XImage* ximage = (::XImage*) (image->structure);

    XPutImage(dpy, drawableXID, gc, ximage,
	      srcX, srcY,
	      destX, destY,
	      width, height);
}
