
package no.rwr.xutil;

import no.rwr.x.*;
//import no.rwr.render.*;
import java.awt.image.*;
import java.awt.Point;
import java.awt.Transparency;
import java.awt.color.*;

public class ImageBuffer {

    ColorModel colorModel;

    /* FIXME: Maybe we shouldn't store this here. Only refer to sample
       model through the raster. */
    SampleModel sampleModel;

    Raster raster;
    XImage ximage;
    Visual visual;
    Pixmap pixmap;
    boolean needFlush = true;
    
    public ColorModel getColorModel() {
	return colorModel;
    }
    
    public Raster getRaster() {
	return raster;
    }

    public XImage getXImage() {
	return ximage;
    }

    public static boolean isCompatible(Visual visual,
				       ColorModel cm, SampleModel sm) {
	if (visual.getDepth() != cm.getPixelSize()) {
	    return false;  // depth mismatch
	}
	// FIXME: What about ignoring alpha channel?


	if (cm instanceof ComponentColorModel) {
	    ComponentColorModel ccm = (ComponentColorModel) cm;
	    ComponentSampleModel csm = (ComponentSampleModel) sm;
	    
	    boolean isDecomposed = true; // FIXME: get this from visual
	    boolean isTrueColor = true; // FIXME: get this from visual

	    int numBands = isDecomposed ? 3 : 1;
	    int[] bandOffsets = csm.getBandOffsets();
	    int sampleSize = csm.getSampleSize(0); // FIXME
	    int[] componentSizes = ccm.getComponentSize();
	    int[] masks = new int[numBands];

	    boolean reverseByteOrder = false; // FIXME: don't assume this
	    /* Maybe we should check both ways? Remember, Xlib is able
	       to reorder bytes on the fly. Depends on whether we want
	       to check if the models will work with a visual, or if
	       we are checking if we can reseat an existing XImage to
	       use data based on the these models. */

	    /* The dilemma here, is that the information discovered
	       here is of use if we want to create an XImage later.
	       For example: if we find that reversing byte order will
	       work, then we need to specify this when creating the
	       XImage.

	       Maybe we should combine these operations to something
	       like: try-to-create-XImage. 
	       
	       Either that, or make this a method that returns an
	       object containing the information discovered, which can
	       be used to create an XImage later. Something like:
	       get-create-XImage-information, and the returned object
	       may say "not-possible".

	       This last approach may allow us to avoid repeating costly
	       calculations, if we intend to create several XImages based
	       on the data with the same models.

	       Possible solution: Make this something that returns an
	       image buffer factory.

	       [This has now been realized. See XImageFactory.]

	       The image buffer factory contain additional attributes
	       that can be queried to find out the circumstances
	       around image buffer
	       creation. Eg. not-possible-due-to-color-model,
	       not-possible-due-to-sample-model,
	       data-must-be-converted, data-must-be-copied,
	       data-will-be-used-directly, etc.

	       The factory can contain methods for creating blank
	       XImages or create XImages based on rasters.

	       A factory can be obtained either by specifying only
	       visual, or visual and ColorModel or visual, ColorModel
	       and SampleModel.
	       
	       Factories can be created for several visuals, and the
	       factories can be queried to find the best, and thereby
	       finding the most suitable visual.  */

	    int bytesPP = sampleSize/8;

	    for (int b=0; b<numBands; b++) {
		int maxValue = (1 << componentSizes[b])-1;
		int offset = bandOffsets[b];
		if (reverseByteOrder) {
		    offset = bytesPP-1 - offset;
		}
		int shift = offset*sampleSize;
		masks[b] = maxValue << shift;
	    }
	    if (isDecomposed) {
		/* FIXME: check that color model matches either direct or
		   true color. */
		
		if (visual.getRedMask()   != masks[0]) return false;
		if (visual.getGreenMask() != masks[1]) return false;
		if (visual.getBlueMask()  != masks[2]) return false;
	    }
	    

	    
	}
	

	return false;
    }
    

    public Pixmap getPixmap() {

	/** Problems will surface when creating a pixmap if the format
	    of the XImage is not accepted by the X server. */

	System.out.println("ximage.getWidth()="+ximage.getWidth());
	System.out.println("ximage.getHeight()="+ximage.getHeight());
	System.out.println("ximage.getDepth()="+ximage.getDepth());
	
	if (pixmap == null) {
	    pixmap = new Pixmap(visual.getScreen().getRootWindow(),
				ximage.getWidth(), ximage.getHeight(),
				visual.getDepth());
	    //				ximage.getDepth());
	}
	System.out.println("pixmap="+pixmap);
	if (needFlush) {
	    // FIXME: don't create a new GC all the time.
	    GC gc = new GC(pixmap);

	    /* If the depth of the image does not match with the depth
	       of the pixmap then a BadMatch will result. */

	    gc.putImage(ximage,
			0, 0,
			0, 0,
			ximage.getWidth(), ximage.getHeight());
	    needFlush = false;
	}
	return pixmap;
    }

    public ImageBuffer(Visual visual, int width, int height) {
	if (visual == null) {
	    throw
		new IllegalArgumentException("visual must not be null");
	}
	this.visual = visual;

	ximage = new XImage(visual, width, height);

	createModels();
	Point location = new Point(0, 0);
	raster = Raster.createWritableRaster(sampleModel, location);

	// FIXME: Don't assume.
	ComponentSampleModel csm = (ComponentSampleModel) sampleModel;

	// Calculate offset to data in the underlying arrays
	int scanlineStride = csm.getScanlineStride();
	int x  =  raster.getMinX() - raster.getSampleModelTranslateX();
	int y  =  raster.getMinY() - raster.getSampleModelTranslateY();

	int numBands = sampleModel.getNumBands();

	DataBuffer dataB = raster.getDataBuffer();

	int offset = 
	    numBands*x + scanlineStride*y +    // from sample model
	    dataB.getOffset();                 // from data buffer

	switch (dataB.getDataType()) {
	case DataBuffer.TYPE_BYTE:
	    ximage.setData(((DataBufferByte) dataB).getData(), offset);
	    break;
	case DataBuffer.TYPE_USHORT:
	    ximage.setData(((DataBufferUShort) dataB).getData(), offset);
	    break;
	default:
	    throw
		new UnsupportedOperationException("Do not know how to " +
						  "set data for data " +
						  "type " +
						  dataB.getDataType());
	}
    }


    /** The main use of this constructor is to create an XImage that
	contains a subregion of the data from another XImage. In this
	case we can assume that Visual and ColorModel match with the
	capabilitites of the X server.

	It might seem that XSubImage would be better for this, but
	XSubImage will allocate new memory, rather than using the
	existing memory.

	We might also want to use this with data from other sources,
	in case the data format by chance is supported by the X
	server. 

	This methods assumes that visual, colorModel and raster match.  */
    public ImageBuffer(Visual visual, ColorModel colorModel, Raster raster) {

	this.colorModel = colorModel;
	this.raster = raster;
	this.visual = visual;

	System.out.println("visual=" +     visual);     // DEBUG
	System.out.println("colorModel=" + colorModel); // DEBUG
	System.out.println("raster=" +     raster);     // DEBUG

	SampleModel sm = raster.getSampleModel();

	System.out.println("sm=" + sm);

	int numBands = sm.getNumBands();

	System.out.println("numBands=" + numBands); // DEBUG

	int depth = colorModel.getPixelSize();
	/* // or from sample model
	int[] bits = sm.getSampleSize();
	for (int b=0; b<numBands; b++)
	    depth += bits[b];
	*/
	if (depth != visual.getDepth()) {
	    throw new IllegalArgumentException("Raster depth does not " +
					       "match visual depth");
	}

	int format = XImage.ZPIXMAP_FORMAT;

	// FIXME: set format based on type of colorModel

	System.out.println("finding coordinates"); // DEBUG

	int x  =  raster.getMinX() - raster.getSampleModelTranslateX();
	int y  =  raster.getMinY() - raster.getSampleModelTranslateY();

	System.out.println("x="+x+", y="+y); // DEBUG

	DataBuffer dataB = raster.getDataBuffer();

	System.out.println("dataB=" + dataB);

	int xOffset = 0;
	int dataOffset = dataB.getOffset();  // from data buffer
	int bytesPerLine = 0;

	/* Change xOffset and dataOffset and bytesPerLine based on x
           and y using knowledge from the sample model. */
	if (sm instanceof ComponentSampleModel) {
	    ComponentSampleModel csm = (ComponentSampleModel) sm;
	    int scanlineStride = csm.getScanlineStride();
	    int bitsPerComponent = 
		DataBuffer.getDataTypeSize(csm.getDataType());

	    // FIXME: deduce format based on pixel stride?

	    /* FIXME: test to ensure that the visual masks match the
	       color/sample model. */

	    // xOffset is unchanged

	    /* We can't use getOffset(x, y) from the sample model
	       since we don't want the band offset added in. */
	    dataOffset += numBands*x + scanlineStride*y;

	    bytesPerLine = scanlineStride*8 / bitsPerComponent;
	} else {
	    throw
		new UnsupportedOperationException("Don't know how to " +
						  "create XImage from " +
						  "sample model " + sm);
	}
	
	// No data. It will be set later...

	int width = raster.getWidth();
	int height = raster.getHeight();
	
	int bitmapPad = 32; // FIXME, don't hardcode this

	System.out.println("creating the XImage");

	ximage = new XImage(visual, depth, format, xOffset,
			    // no data
			    width, height,
			    bitmapPad,
			    bytesPerLine);
	
	System.out.println("ximage=" + ximage);

	switch (dataB.getDataType()) {
	case DataBuffer.TYPE_BYTE:
	    ximage.setData(((DataBufferByte) dataB).getData(),
			   dataOffset);
	    break;
	case DataBuffer.TYPE_USHORT:
	    ximage.setData(((DataBufferUShort) dataB).getData(), 
			   dataOffset);
	    break;
	default:
	    throw
		new UnsupportedOperationException("Do not know how to " +
						  "set data for data " +
						  "type " +
						  dataB.getDataType());
	}

	System.out.println("data set");

    }
    


    protected void createModels() {
	int bitsPP = ximage.getBitsPerPixel();
	
    tryByteComponentModel:
	if ((bitsPP%8) == 0) {
	    System.out.println("pixels are byte multiples"); // DEBUG
	    
	    
	    int ri = getLSBFirstByteOffsetFromMask(ximage.getRedMask());
	    if (ri == -1) break tryByteComponentModel;
	    int gi = getLSBFirstByteOffsetFromMask(ximage.getGreenMask());
	    if (gi == -1) break tryByteComponentModel;
	    int bi = getLSBFirstByteOffsetFromMask(ximage.getBlueMask());
	    if (bi == -1) break tryByteComponentModel;
	    System.out.println("samples are bytes"); // DEBUG

	    int bytesPP = bitsPP / 8;
	    
	    boolean reverseByteOrder = false; // FIXME: don't assume this
	    if (reverseByteOrder) {
		ri = bytesPP-1 - ri;
		gi = bytesPP-1 - gi;
		bi = bytesPP-1 - bi;
	    }
	    int[] bandOffsets = { ri, gi, bi };

	    int[] bits = { 8, 8, 8 };

	    // FIXME: ensure that bytePP match with color space.

	    ColorSpace sRGBCS =
		ColorSpace.getInstance(ColorSpace.CS_sRGB);
	    
	    int dataType = DataBuffer.TYPE_BYTE;

	    /* FIXME: We need sanity checking to ensure that color
	       model and sample model are compatible. */

	    colorModel =
		new ComponentColorModel(sRGBCS,
					bits,
					false, // does not have alpha
					false, // no premultiplied alpha
					Transparency.OPAQUE,
					dataType);
	    sampleModel = 
		new ComponentSampleModel(dataType,
					 ximage.getWidth(),
					 ximage.getHeight(),
					 bytesPP,
					 ximage.getBytesPerLine(),
					 bandOffsets);
	}
	
	throw new UnsupportedOperationException("Don't know how to " +
						"create models for " +
						" ximage " + ximage);
    }


    private int getLSBFirstByteOffsetFromMask(int mask) {
	switch (mask) {
	case 0xff:
	    return 0;
	case 0xff00:
	    return 1;
	case 0xff0000:
	    return 2;
	case 0xff000000:
	    return 3;
	default:
	    // FIXME: maybe throw exception instead?
	    return -1; 
	}
    }
}
